#!/usr/bin/env python3
import os
import sys
import urllib3
import argparse
import configparser

from cnaas_sync.nav import NAV
from cnaas_sync.ni import NI
from cnaas_sync.nms import NMS



# Disable Unverified HTTPS request warning
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def get_args():
    """
    This function used argpase to parse arguments from stdin and convert it to simple manipulable key-value pairs
    used in the run function
    :return:
    """
    parser = argparse.ArgumentParser(
        prog="cnaas-sync-cli",
        description='Scripts used for syncing the switches inventory database between NI and NAV with data from NMS',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('-t',
                        '--target',
                        choices=['nav', 'ni'],
                        metavar='str',
                        required=True,
                        help='Target subject of the sync operation')

    parser.add_argument('-cf',
                        '--config-file',
                        metavar='file',
                        required=True,
                        type=argparse.FileType('r'),
                        help='Configuration file containing key and values pair for optional parameters')

    parser.add_argument('-nvu',
                        '--nav-base-url',
                        help='Nav Api base url',
                        metavar='str',
                        default=None,
                        type=str)

    parser.add_argument('-nvk',
                        '--nav-api-key',
                        help='Nav api key',
                        metavar='str',
                        default=None,
                        type=str)

    parser.add_argument('-niu',
                        '--ni-base-url',
                        help='Nav Api base url',
                        metavar='str',
                        default=None,
                        type=str)

    parser.add_argument('-niuser',
                        '--ni-user-name',
                        help='Ni username generated from the django admin or postgres db',
                        metavar='str',
                        default=None,
                        type=str)

    parser.add_argument('-nik',
                        '--ni-api-key',
                        help='Ni user api-key from django admin',
                        metavar='str',
                        default=None,
                        type=str)

    parser.add_argument('-nmsu',
                        '--nms-base-url',
                        help='NMS api base url',
                        metavar='str',
                        default=None,
                        type=str)

    parser.add_argument('-nmsjwt',
                        '--nms-jwt-token',
                        help='NMS jwt token',
                        metavar='str',
                        default=None,
                        type=str)

    return parser.parse_args()


def coalesce(value, alternative):
    return alternative if value is None else value


def handle_param_errors(param_dict):
    for key, param in zip(param_dict.keys(), param_dict.values()):
        if len(str(param).strip()) == 0:
            key = str(key).replace("_", "-")
            return True, f"Parameter {key} should not be empty. Update the config file or set it with appropriate flag."
    return False, None


def update_ni(args, config):
    # Parameters init
    ni_conf = config['NI']
    nms_conf = config['NMS']
    ni_base_url = coalesce(args.ni_base_url, ni_conf['NI_BASE_URL'])
    ni_user_name = coalesce(args.ni_user_name, ni_conf['NI_USERNAME'])
    ni_api_key = coalesce(args.ni_api_key, ni_conf['NI_USER_API_KEY'])
    nms_base_url = coalesce(args.nms_base_url, nms_conf['NMS_BASE_URL'])
    nms_jwt_token = coalesce(args.nms_jwt_token, nms_conf['NMS_JWT_TOKEN'])
    param_dict = {'ni_base_url': ni_base_url,
                  'ni_user_name': ni_user_name,
                  'ni_api_key': ni_api_key,
                  'nms_base_url': nms_base_url,
                  'nms_jwt_token': nms_jwt_token}
    error, msg = handle_param_errors(param_dict)
    if error:
        raise Exception(msg)

    # System update
    ni = NI(ni_base_url, ni_user_name, ni_api_key)
    nms = NMS(nms_base_url, nms_jwt_token)
    nms_devices = nms.get_all_devices()

    # Update NI
    ni.update(nms_devices)


def update_nav(args, config):
    # Parameters init
    nav_conf = config['NAV']
    ni_conf = config['NI']
    nms_conf = config['NMS']
    nav_base_url = coalesce(args.nav_base_url, nav_conf['NAV_BASE_URL'])
    nav_api_key = coalesce(args.nav_api_key, nav_conf['NAV_API_KEY'])
    ni_base_url = coalesce(args.ni_base_url, ni_conf['NI_BASE_URL'])
    ni_user_name = coalesce(args.ni_user_name, ni_conf['NI_USERNAME'])
    ni_api_key = coalesce(args.ni_api_key, ni_conf['NI_USER_API_KEY'])
    nms_base_url = coalesce(args.nms_base_url, nms_conf['NMS_BASE_URL'])
    nms_jwt_token = coalesce(args.nms_jwt_token, nms_conf['NMS_JWT_TOKEN'])
    param_dict = {'nav_base_url': nav_base_url,
                  'nav_api_key': nav_api_key,
                  'ni_base_url': ni_base_url,
                  'ni_user_name': ni_user_name,
                  'ni_api_key': ni_api_key,
                  'nms_base_url': nms_base_url,
                  'nms_jwt_token': nms_jwt_token}
    error, msg = handle_param_errors(param_dict)
    if error:
        raise Exception(msg)

    # System update
    ni = NI(ni_base_url, ni_user_name, ni_api_key)
    nms = NMS(nms_base_url, nms_jwt_token)
    nav = NAV(nav_base_url, nav_api_key)
    ni_devices = ni.get_all_devices()

    # Update NAV
    nav.update(ni_devices, nms)


def run():
    args = get_args()
    config = configparser.ConfigParser()

    # Required arguments
    target_system = args.target
    config_file = args.config_file
    config.read(config_file.name)

    # Update system
    if target_system == "ni":
        update_ni(args, config)
    elif target_system == "nav":
        update_nav(args, config)


if __name__ == "__main__":
    run()
